//
//  AfficheurController.m
//  Afficheur
//
//  Created by kichi on 08/04/05.
//  Copyright 2008 Katsuhiko Ichinose. All rights reserved.
//

#import "AfficheurController.h"
#import "HotKeyedApplication.h"
#import "TextFieldHUD.h"
#import "TwitterAPI.h"
#import "JaikuAPI.h"
#import "TumblrAPI.h"
#import "GrowlNotifier.h"
#import "TinyURL.h"
#import <OgreKit/OgreKit.h>

@implementation AfficheurController

NSString*	Afficheur					= @"Afficheur";

NSString*	GrowlAfficheurSuccessNotify	= @"Post was successful";
NSString*	GrowlAfficheurFailureNotify	= @"Post was failure";

- (AfficheurController *)init
{
	self = [super init];
	if (self)
	{
		_prefs = [[[AfficheurPreferences alloc] init] retain];
		_leopard = NO;
		_hotKeyRef = nil;
		_textViewHUD = [[[TextViewHUD alloc] initWithController:self] retain];
		if (_textViewHUD)
		{
			[_textViewHUD setDelegate:self];
		}
		
		SInt32 gestalt;
		if (!Gestalt(gestaltSystemVersion, &gestalt))
		{
			if (gestalt >= 0x1050)
			{
				_leopard = YES;
			}
		}
		_doCommand = 0;
	}
	return self;
}

- (void)dealloc
{
	[_prefs release];
	[_textViewHUD release];
	[super dealloc];
}

- (void)unregisterHotKey
{
	if (_hotKeyRef)
	{
		[[HotKeyedApplication sharedApplication] unregisterHotKey:_hotKeyRef];
		_hotKeyRef = nil;
	}
}

- (void)registerHotKey
{
	UInt32 keyCode = [_prefs generalHotKey];
	[self unregisterHotKey];
	if (keyCode != -1)
	{
		_hotKeyRef = [[HotKeyedApplication sharedApplication] registerHotKey:[KeyEquivView keyCodeForKeyCode:keyCode]
																withModifier:[KeyEquivView modifierKeyForKeyCode:keyCode]
																	  target:self
																	selector:@selector(hotKey)];
	}
}

- (BOOL) isURL:(NSString *)string
{
	return ([string rangeOfRegularExpressionString:@"(?:http|https)://"].length != 0);
}

- (void)performThread:(SEL)selector withStatus:(NSString *)status
{
#if 0
	if (_leopard && [self respondsToSelector:@selector(performSelectorInBackground:withObject:)])
	{
		[self performSelectorInBackground:selector
							   withObject:status];
	}
	else
	{
#endif
		[NSThread detachNewThreadSelector:selector
								 toTarget:self
							   withObject:status];
#if 0
	}
#endif
}

- (void)performPostFinish:(int)ng
{
	[_text setEnabled:YES];
//	[_spinner setImage:nil];
	if (!ng)
	{
		[_panel orderOut:self];
	}
}

- (void)performPostOK
{
	_postCount--;
	if (_postCount == 0)
	{
		[self performPostFinish:_postNG];
	}
}

- (void)performPostNG
{
	_postNG++;
	[self performPostOK];
}

- (void)postOK
{
	[self performSelectorOnMainThread:@selector(performPostOK)
						   withObject:nil
						waitUntilDone:NO];
}

- (void)postNG
{
	[self performSelectorOnMainThread:@selector(performPostNG)
						   withObject:nil
						waitUntilDone:NO];
}

- (void)performNotify:(id)object
{
	if ([object isKindOfClass:[NSArray class]] && ([object count] >= 3))
	{
		[_growl notifyWithTitle:[object objectAtIndex:0]
					description:[object objectAtIndex:1]
			   notificationName:[object objectAtIndex:2]
					   iconData:([object count] >= 4 ? [object objectAtIndex:3] : nil)
					   priority:0
					   isSticky:false
				   clickContext:nil];
	}
}

- (void)notifyWithTitle:(NSString *)title
			description:(NSString *)description
	   notificationName:(NSString *)notificationName
			   iconData:(NSData *)iconData
{
	[_growl notifyWithTitle:title
				description:description
		   notificationName:notificationName
				   iconData:iconData
				   priority:0
				   isSticky:false
			   clickContext:nil];
}

- (NSString *)determineResponse:(NSArray *)response
{
	NSString* description = @"Post was successful.";
	if ([response count] > ResultDescription)
	{
		id res = [response objectAtIndex:ResultDescription];
		if (![res isKindOfClass:[NSNull class]])
		{
			description = [NSString stringWithFormat:@"%@", res];
		}
	}
	return description;
}

- (NSString *)determineNotify:(NSArray *)response
{
	NSString* notify = GrowlAfficheurSuccessNotify;
	if ([response count] > ResultDescription)
	{
		id res = [response objectAtIndex:ResultDescription];
		if (![res isKindOfClass:[NSNull class]])
		{
			notify = GrowlAfficheurFailureNotify;
		}
	}
	return notify;
}

- (BOOL)determineSuccess:(NSArray *)response
{
	BOOL success = YES;
	if ([response count] > ResultDescription)
	{
		id res = [response objectAtIndex:ResultDescription];
		if (![res isKindOfClass:[NSNull class]])
		{
			success = NO;
		}
	}
	return success;
}

- (void)performDeterminePost:(id)object
{
	if ([object isKindOfClass:[NSArray class]] && ([object count] >= 2))
	{
		//NSLog(@"[%@ performDeterminePost] %@", [self className], [object objectAtIndex:1]);
		[self notifyWithTitle:[object objectAtIndex:1]
				  description:[self determineResponse:[object objectAtIndex:0]]
			 notificationName:[self determineNotify:[object objectAtIndex:0]]
					 iconData:nil];
		if ([self determineSuccess:[object objectAtIndex:0]])
		{
			[self postOK];
		}
		else
		{
			[self postNG];
		}
	}
}

- (void)determinePost:(NSArray *)response
			withTitle:(NSString *)title
{
	[self performSelectorOnMainThread:@selector(performDeterminePost:)
						   withObject:[NSArray arrayWithObjects:response, title, nil]
						waitUntilDone:NO];
}

- (void)postTwitter:(NSString *)status
{
	[self performThread:@selector(performPostTwitter:)
			 withStatus:status];
}

- (void)performPostTwitter:(NSString *)status
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	TwitterAPI *api = [[[TwitterAPI alloc] initWithObjectsAndKeys:
						[_prefs accountTwitterUser], TwitterUser,
						[_prefs accountTwitterPass], TwitterPass,
						Afficheur, TwitterSource,
						nil] autorelease];
	NSArray *res = [api update:status];
	[self determinePost:res
			  withTitle:@"Post to Twitter"];
	[pool release];
}

- (void)postJaiku:(NSString *)status
{
	[self performThread:@selector(performPostJaiku:)
			 withStatus:status];
}

- (void)performPostJaiku:(NSString *)status
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	JaikuAPI *api = [[[JaikuAPI alloc] initWithObjectsAndKeys:
					  [_prefs accountJaikuUser], JaikuUser,
					  [_prefs accountJaikuPass], JaikuPass,
					  nil] autorelease];
	NSArray *res = [api update:status withLocation:[_prefs accountJaikuLocation]];
	[self determinePost:res
			  withTitle:@"Post to Jaiku"];
	[pool release];
}

- (void)postTumblr:(NSString *)status
{
	[self performThread:@selector(performPostTumblr:)
			 withStatus:status];
}

- (void)performPostTumblr:(NSString *)status
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	if ([self isURL:status])
	{
		NSString *url = nil;
		OGRegularExpression *regex = [OGRegularExpression
									  regularExpressionWithString:@"((?:http|https)://\\S+)"
									  options:OgreIgnoreCaseOption | OgreMultilineOption];
		// マッチ結果の列挙子の生成
		OGRegularExpressionMatch *match = [regex matchInString:status];
		NSString *substr = [match matchedString];
		if (substr)
		{
			url = substr;
		}
		regex = [OGRegularExpression
				 regularExpressionWithString:@"^(.*)\\s+(?:http|https)://"
				 options:OgreIgnoreCaseOption | OgreMultilineOption];
		// マッチ結果の列挙子の生成
		match = [regex matchInString:status];
		NSString *title = [match substringAtIndex:1];
		if (url)
		{
			TumblrAPI *api = [[[TumblrAPI alloc] initWithObjectsAndKeys:
							   [_prefs accountTumblrUser], TumblrUser,
							   [_prefs accountTumblrPass], TumblrPass,
							   Afficheur, TumblrGenerator,
							   nil] autorelease];
			NSArray *res = [api createLink:url title:title];
			[self determinePost:res
					  withTitle:@"Post to Tumblr"];
		}
	}
	[pool release];
}

- (void)postWassr:(NSString *)status
{
	[self performThread:@selector(performPostWassr:)
			 withStatus:status];
}

- (void)performPostWassr:(NSString *)status
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	TwitterAPI *api = [[[TwitterAPI alloc] initWithObjectsAndKeys:
						@"api.wassr.jp", TwitterHost,
						@"http://api.wassr.jp/statuses", TwitterURL,
						[_prefs accountWassrUser], TwitterUser,
						[_prefs accountWassrPass], TwitterPass,
						Afficheur, TwitterSource,
						nil] autorelease];
	NSArray *res = [api update:status];
	[self determinePost:res
			  withTitle:@"Post to Wassr"];
	[pool release];
}

- (void)postNowa:(NSString *)status
{
	[self performThread:@selector(performPostNowa:)
			 withStatus:status];
}

- (void)performPostNowa:(NSString *)status
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	TwitterAPI *api = [[[TwitterAPI alloc] initWithObjectsAndKeys:
						@"api.nowa.jp", TwitterHost,
						@"http://api.nowa.jp/statuses", TwitterURL,
						[_prefs accountNowaUser], TwitterUser,
						[_prefs accountNowaPass], TwitterPass,
						nil] autorelease];
	NSArray *res = [api update:status];
	[self determinePost:res
			  withTitle:@"Post to nowa"];
	[pool release];
}

- (void)postFrepa:(NSString *)status
{
	[self performThread:@selector(performPostFrepa:)
			 withStatus:status];
}

- (void)performPostFrepa:(NSString *)status
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	TwitterAPI *api = [[[TwitterAPI alloc] initWithObjectsAndKeys:
						@"www.frepa.livedoor.com", TwitterHost,
						@"http://www.frepa.livedoor.com/statuses", TwitterURL,
						[_prefs accountFrepaUser], TwitterUser,
						[_prefs accountFrepaPass], TwitterPass,
						nil] autorelease];
	NSArray *res = [api update:status];
	[self determinePost:res
			  withTitle:@"Post to frepa"];
	[pool release];
}

- (void)setTextString:(NSString *)string
{
	NSTextField *textField = _text;
	[textField setStringValue:string];
	[self controlTextDidChange:[NSDictionary dictionaryWithObjectsAndKeys:
								textField, @"object", nil]];
}

- (NSString *)appleScript:(NSString *)script
{
	NSString *result = nil;
	NSDictionary* errorDict;
	NSAppleEventDescriptor* returnDescriptor = nil;
	NSAppleScript* scriptObject = [[NSAppleScript alloc] initWithSource:script];
	returnDescriptor = [scriptObject executeAndReturnError: &errorDict];
	[scriptObject release];
	if (returnDescriptor)
	{
		result = [returnDescriptor stringValue];
	}
	else
	{
		NSLog(@"[%@ appleScript] ERROR:%@", [self className], errorDict);
	}
	return result;
}

- (NSString *)titleOfSafari
{
	return [self appleScript:@""\
			"tell application \"Safari\"\n"\
			"set aTitle to (get name of document 1)\n"\
			"end tell"];
}

- (NSString *)URLOfSafari
{
	return [self appleScript:@""\
			"tell application \"Safari\"\n"\
			"set aURL to (get URL of document 1)\n"\
			"end tell"];
}

- (NSString *)titleAndURLOfSafari
{
	NSString *title = [self titleOfSafari];
	NSString *URL = [self URLOfSafari];
	if (title && URL)
	{
		return [NSString stringWithFormat:@"%@ %@", title, URL];
	}
	else if (title)
	{
		return title;
	}
	else if (URL)
	{
		return URL;
	}
	return nil;
}

- (NSString *)titleOfFirefox
{
	return [self appleScript:@""\
			"tell application \"Firefox\"\n"\
			"set theTitle to get «class pTit» of window 1\n"\
			"end tell"];
}

- (NSString *)URLOfFirefox
{
	return [self appleScript:@""\
			"tell application \"Firefox\"\n"\
			"set theURL to «class curl» of window 1\n"\
			"end tell"];
}

- (NSString *)titleAndURLOfFirefox
{
	NSString *title = [self titleOfFirefox];
	NSString *URL = [self URLOfFirefox];
	if (title && URL)
	{
		return [NSString stringWithFormat:@"%@ %@", title, URL];
	}
	else if (title)
	{
		return title;
	}
	else if (URL)
	{
		return URL;
	}
	return nil;
}

- (void)grabURL
{
	if ([_prefs generalGrabURL])
	{
		NSWorkspace *ws = [NSWorkspace sharedWorkspace];
		if (ws)
		{
			NSDictionary *ap = [ws activeApplication];
			if (ap)
			{
				NSString *appName = [ap valueForKey:@"NSApplicationName"];
				if (appName)
				{
					NSString *text = [_prefs generalTextURL];
					if (![text isEqualToString:@""])
					{
						text = [NSString stringWithFormat:@"%@ ", text];
					}
					if ([appName compare:@"safari"
								 options:NSCaseInsensitiveSearch] == NSOrderedSame)
					{
						NSString *title = [self titleAndURLOfSafari];
						if (title)
						{
							[self setTextString:[NSString stringWithFormat:@"%@%@", text, title]];
						}
					}
					if ([appName compare:@"firefox"
								 options:NSCaseInsensitiveSearch] == NSOrderedSame)
					{
						NSString *title = [self titleAndURLOfFirefox];
						if (title)
						{
							[self setTextString:[NSString stringWithFormat:@"%@%@", text, title]];
						}
					}
				}
			}
		}
	}
}

- (void)grabITunes
{
	if ([_prefs generalGrabITunes])
	{
		NSWorkspace *ws = [NSWorkspace sharedWorkspace];
		if (ws)
		{
			NSDictionary *ap = [ws activeApplication];
			if (ap)
			{
				NSString *appName = [ap valueForKey:@"NSApplicationName"];
				if (appName)
				{
					if ([appName compare:@"iTunes"
								 options:NSCaseInsensitiveSearch] == NSOrderedSame)
					{
						NSString *text = [_prefs generalTextITunes];
						NSString *title = nil;
						NSString *artist = nil;
						NSString *album = nil;
						NSString *iTunes = nil;
						if (![text isEqualToString:@""])
						{
							text = [NSString stringWithFormat:@"%@ ", text];
						}
						title = [self appleScript:@""\
								 "tell application \"iTunes\"\n"\
								 "set theTitle to name of current track --曲名\n"\
								 "end tell"];
						artist = [self appleScript:@""\
								  "tell application \"iTunes\"\n"\
								  "set theArtist to artist of current track --演奏者\n"\
								  "end tell"];
						album = [self appleScript:@""\
								 "tell application \"iTunes\"\n"\
								 "set theAlbum to album of current track --演奏者\n"\
								 "end tell"];
						if (title && ![title isEqualToString:@""] &&
							artist && ![artist isEqualToString:@""] &&
							album && ![album isEqualToString:@""])
						{
							iTunes = [NSString stringWithFormat:@"%@ -- %@ [%@]", title, artist, album];
						}
						else if (title && ![title isEqualToString:@""] &&
								 artist && ![artist isEqualToString:@""])
						{
							iTunes = [NSString stringWithFormat:@"%@ -- %@", title, artist];
						}
						else if (title && ![title isEqualToString:@""])
						{
							iTunes = title;
						}
						else if (artist && ![artist isEqualToString:@""])
						{
							iTunes = artist;
						}
						if (iTunes)
						{
							[self setTextString:[NSString stringWithFormat:@"%@%@", text, iTunes]];
						}
					}
				}
			}
		}
	}
}

- (NSString *)encodeTinyURL:(NSString *)string
			 withCreateFlag:(BOOL)flag
{
	NSString *jpn = @"℀-➾　-㏾一-龥豈-鶴！-￬";
	NSString *anc = @"0-9a-zA-Z\\-_\\.!~\\*'\\(\\)\\/\\?&=,:%#";
	NSString *lst = @"0-9a-zA-Z\\-_\\.!~\\*'\\/";
    NSString *exp = [NSString stringWithFormat:@"((?:http|https)://[%@%@]*[%@])", jpn, anc, lst];
	OGRegularExpression *regex = [OGRegularExpression
								  regularExpressionWithString:exp
								  options:OgreMultilineOption];
	// マッチ結果の列挙子の生成
	NSEnumerator    *enumerator = [regex matchEnumeratorInString:string];
	OGRegularExpressionMatch    *match;	// マッチ結果
	while ((match = [enumerator nextObject]) != nil) {	// 順番にマッチ結果を得る
		NSString *url = [match matchedString];
		NSString *tiny = @"http://tinyurl.com/123456";
		if (flag)
		{
			tiny = [TinyURL create:url];
		}
		if (tiny)
		{
			OGRegularExpression *replace = [OGRegularExpression
											regularExpressionWithString:[OGRegularExpression regularizeString:url]
											options:OgreMultilineOption];
			string = [replace replaceFirstMatchInString:string
											 withString:tiny];
		}
	}
	return string;
}

- (NSString *)encodeTinyURL:(NSString *)string
{
	return [self encodeTinyURL:string withCreateFlag:YES];
}

- (void)doAfficheur:(NSString *)status
   altEncodeTinyURL:(BOOL)altEncodeTinyURL
{
	NSString *encode = [NSString stringWithString:status];
	BOOL encodeTinyURL = [_prefs generalEncodeTinyURL];
	if (altEncodeTinyURL)
	{
		encodeTinyURL = !encodeTinyURL;
	}
	if (encodeTinyURL)
	{
		encode = [self encodeTinyURL:encode];
	}
	//NSLog(@"doAfficheur: \'%@\'", encode);
	//return;
	if (![status isEqual:@""])
	{
		_postCount = 0;
		_postNG = 0;
		if ([_prefs postTwitter])
		{
			_postCount++;
			[self postTwitter:encode];
		}
		if ([_prefs postJaiku])
		{
			_postCount++;
			[self postJaiku:encode];
		}
		if ([_prefs postTumblr])
		{
			_postCount++;
			[self postTumblr:status];
		}
		if ([_prefs postWassr])
		{
			_postCount++;
			[self postWassr:encode];
		}
		if ([_prefs postNowa])
		{
			_postCount++;
			[self postNowa:encode];
		}
		if ([_prefs postFrepa])
		{
			_postCount++;
			[self postFrepa:encode];
		}
		if (_postCount)
		{
			[_text setEnabled:NO];
		//	[_spinner setImage:[NSImage imageNamed:@"spinner_16"]];
		//	[_spinner setAnimates:YES];
		}
	}
}

// Loaded from NIB

- (void)awakeFromNib
{
	[_window setAlphaValue:0.0];
	NSRect screen = [[NSScreen mainScreen] frame];
	NSRect frame = [_panel frame];
	frame.origin.x = (screen.size.width - frame.size.width) / 2 + screen.origin.x;
	frame.origin.y = (screen.size.height - frame.size.height) / 4 * 3 + screen.origin.y;
	[_panel setFrame:frame display:true];
}

// Application Delegate

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	NSStatusBar*	statusBar = [NSStatusBar systemStatusBar];
	NSStatusItem*	statusItem = [[statusBar statusItemWithLength:24/*NSVariableStatusItemLength*/] retain];
	NSImage* icon = [NSImage imageNamed:@"MenuIcon"];
	[statusItem setTitle:nil];
	[statusItem setImage:icon];
	[statusItem setHighlightMode:YES];
	[statusItem setMenu:_menu];
	[statusItem setEnabled:YES];
	[_growl initWithDelegate:self];
	[_growl startWithApplicationName:@"Afficheur"
					   notifications:[NSArray arrayWithObjects:
									  GrowlAfficheurSuccessNotify,
									  GrowlAfficheurFailureNotify,
									  nil]
				dafaultNotifications:nil
								icon:nil];
	[self registerHotKey];
	[NSApp activateIgnoringOtherApps:YES];
}

- (void)applicationWillTerminate:(NSNotification *)aNotification
{
	[_prefs applicationWillTerminate];
}

- (void)applicationDidBecomeActive:(NSNotification *)aNotification
{
	[self onOpen:self];
}

// Growl Delegate

- (void)growlIsReady
{
}

- (void)growlNotificationWasClicked:(id)clickContext
{
}

- (void)growlNotificationTimedOut:(id)clickContext
{
}

// TextField Delegate

- (void)controlTextDidChange:(NSNotification *)aNotification
{
	NSString* txt = [_text stringValue];
	if (aNotification)
	{
		txt = [[aNotification valueForKey:@"object"]  stringValue];
	}
	NSString* opt = @"";
	if ([_prefs generalEncodeTinyURL] && [self isURL:txt])
	{
		NSString *tiny = [self encodeTinyURL:txt withCreateFlag:NO];
		opt = [NSString stringWithFormat:@" (%u / %uB)",
		 [tiny length],
		 [tiny lengthOfBytesUsingEncoding:NSUTF8StringEncoding]
		 ];
	}
	[_label setStringValue:
	 [NSString stringWithFormat:@"%u / %uB%@",
	  [txt length],
	  [txt lengthOfBytesUsingEncoding:NSUTF8StringEncoding],
	  opt
	 ]];
}

- (void)enterKeyDown:(id)object
{
	if ([object isKindOfClass:[NSNumber class]])
	{
		_doCommand = 1;
	}
}

- (void)enterKeyUp:(id)object
{
	if ([object isKindOfClass:[NSNumber class]])
	{
		int postTo = [_prefs generalPostTo];
		int postToALT = [_prefs generalPostToALT];
		int	modifiers = [object intValue] &	(NSShiftKeyMask | NSControlKeyMask | NSAlternateKeyMask| NSCommandKeyMask);
		NSString* text = [[_textViewHUD textStorage] string];
		//NSLog(@"enterKeyUp: %08X : %d \"%@\"", modifiers, _doCommand, text);
		if (_doCommand == 2)
		{
			BOOL	doAfficheur = NO;
			BOOL	altEncodeTinyURL = NO;
			switch (modifiers)
			{
				case 0:
					if (postTo == PostToEnter)
					{
						doAfficheur = YES;
					}
					else if (postToALT == PostToEnter)
					{
						doAfficheur = YES;
						altEncodeTinyURL = YES;
					}
					break;
				case NSControlKeyMask:
					if (postTo == PostToControlEnter)
					{
						doAfficheur = YES;
					}
					else if (postToALT == PostToControlEnter)
					{
						doAfficheur = YES;
						altEncodeTinyURL = YES;
					}
					break;
				case NSAlternateKeyMask:
					if (postTo == PostToOptionEnter)
					{
						doAfficheur = YES;
					}
					else if (postToALT == PostToOptionEnter)
					{
						doAfficheur = YES;
						altEncodeTinyURL = YES;
					}
					break;
				case NSShiftKeyMask:
					if (postTo == PostToShiftEnter)
					{
						doAfficheur = YES;
					}
					else if (postToALT == PostToShiftEnter)
					{
						doAfficheur = YES;
						altEncodeTinyURL = YES;
					}
					break;
				case NSControlKeyMask | NSShiftKeyMask:
					if (postTo == PostToControlShiftEnter)
					{
						doAfficheur = YES;
					}
					else if (postToALT == PostToControlShiftEnter)
					{
						doAfficheur = YES;
						altEncodeTinyURL = YES;
					}
					break;
				case NSAlternateKeyMask | NSShiftKeyMask:
					if (postTo == PostToOptionShiftEnter)
					{
						doAfficheur = YES;
					}
					else if (postToALT == PostToOptionShiftEnter)
					{
						doAfficheur = YES;
						altEncodeTinyURL = YES;
					}
					break;
			}
			if (doAfficheur)
			{
				[self doAfficheur:text altEncodeTinyURL:altEncodeTinyURL];
			}
		}
	}
	_doCommand = 0;
}

- (BOOL)control:(NSControl *)control
		 textView:(NSTextView *)textView
doCommandBySelector:(SEL)command
{
	NSString *cmd = NSStringFromSelector(command);
	if (cmd && (textView == _textViewHUD))
	{
		if (_doCommand == 1)
		{
			_doCommand = 2;
			return YES;
		}
	}
	return NO;
}

// IBAction

- (IBAction)onOpen:(id)sender
{
	[self grabURL];
	[self grabITunes];
	[self controlTextDidChange:nil];
	[_panel makeKeyAndOrderFront:self];
	[NSApp activateIgnoringOtherApps : YES];
}

- (void)endPreferences
{
	[self registerHotKey];
}

- (IBAction)onPreferences:(id)sender
{
	[NSApp activateIgnoringOtherApps : YES];
	[self unregisterHotKey];
	[_prefs doDialogWithWindow:_panel selector:@selector(endPreferences) target:self];
}

- (IBAction)onAction:(id)sender
{
	[self doAfficheur:[sender stringValue] altEncodeTinyURL:NO];
}

// HotKey Action

- (void)hotKey
{
	[self onOpen:self];
}

// Window Delegate

- (id)windowWillReturnFieldEditor:(NSWindow *)sender toObject:(id)anObject
{
	if ([anObject isKindOfClass:[TextFieldHUD class]])
	{
		[_textViewHUD setupSelectedTextAttributes];
		return _textViewHUD;
	}
	return nil;
}

@end
