//
//  JaikuAPI.m
//  Afficheur
//
//  Created by kichi on 08/04/06.
//  Copyright 2008 Katsuhiko Ichinose. All rights reserved.
//

#import "JaikuAPI.h"
#import <JSON/JSON.h>
#import <OgreKit/OgreKit.h>

NSString *JaikuUser	= @"JaikuUser";
NSString *JaikuPass	= @"JaikuPass";
NSString *JaikuHost	= @"JaikuHost";
NSString *JaikuURL	= @"JaikuURL";

@implementation JaikuAPI

- (JaikuAPI *)init
{
	self = [super init];
	if (self)
	{
		_keys = [NSDictionary dictionaryWithObjectsAndKeys:
				 @"", JaikuUser,
				 @"", JaikuPass,
				 @"", JaikuHost,
				 @"", JaikuURL,
				 nil];
		_dic = [[[NSMutableDictionary alloc] initWithObjectsAndKeys:
				 @"", JaikuUser,
				 @"", JaikuPass,
				 @"api.jaiku.com", JaikuHost,
				 @"http://api.jaiku.com", JaikuURL,
				 nil] retain];
	}
	return self;
}

- (void)dealloc
{
	[_dic release];
	[super dealloc];
}

- (JaikuAPI *)initWithObjectsAndKeys:(id)firstObject, ...
{
	[self init];
	id obj = firstObject;
	va_list	ap;
	va_start(ap, firstObject);
	while (obj)
	{
		id key = va_arg(ap, id);
		if (key == nil)
		{
			break;
		}
		id val = [_keys valueForKey:key];
		if (val == nil)
		{
			NSLog(@"[%@ initWithObjectsAndKeys] NoKey: %@", [self className], key);
		}
		else
		{
			[_dic setValue:obj forKey:key];
		}
		obj = va_arg(ap, id);
	}
	va_end(ap);
	return self;
}

- (id)postWithAuth:(NSString *)path
			header:(NSMutableDictionary *)aHeader
			  body:(NSString *)aBody
{
	[aHeader setValue:[_dic valueForKey:JaikuHost] forKey:@"Host"];
	return [self post:[NSString stringWithFormat:@"%@/%@", [_dic valueForKey:JaikuURL], path]
			   header:aHeader
				 body:[NSString stringWithFormat:@"user=%@&personal_key=%@&%@",
					   [_dic valueForKey:JaikuUser],
					   [_dic valueForKey:JaikuPass],
					   aBody]];
}

- (NSString *)determineError:(NSError *)error
					response:(NSURLResponse *)response
					  result:(id)result
{
	NSString *description = nil;
	if (![error isKindOfClass:[NSNull class]])
	{
		NSDictionary *dic = [error userInfo];
		NSString *desc = [dic valueForKey:@"NSLocalizedDescription"];
		if (desc && [desc isKindOfClass:[NSString class]] && ![desc isEqualToString:@""])
		{
			description = [NSString stringWithFormat:@"Post was failure.\n(%@)", desc];
		}
	}
	else if ([result isKindOfClass:[NSDictionary class]])
	{
		//NSString *status = [result valueForKey:@"status"];
		NSString *message = [result valueForKey:@"message"];
		if (message)
		{
			description = [NSString stringWithFormat:@"Post was failure.\n(%@)", message];
		}
	}
	else if ([result isKindOfClass:[NSString class]])
	{
		if ([result rangeOfRegularExpressionString:@"^<!DOCTYPE HTML"
										   options:OgreIgnoreCaseOption].length ||
			[result rangeOfRegularExpressionString:@"^<HTML"
										   options:OgreIgnoreCaseOption].length)
		{
			// 正規表現オブジェクトの生成
			OGRegularExpression *regex = [OGRegularExpression
										  regularExpressionWithString:@"<TITLE>(.*)</TITLE>$"
										  options:OgreIgnoreCaseOption | OgreMultilineOption];
			// マッチ結果の列挙子の生成
			OGRegularExpressionMatch *match = [regex matchInString:result];
			NSString *title = [match substringAtIndex:1];
			if (title && ![title isEqualToString:@""])
			{
				result = title;
			}
		}
		description = [NSString stringWithFormat:@"Post was failure.\n(%@)", result];
	}
	if (description)
	{
		NSLog(@"[%@ determineResponse] %@", [self className], description);
	}
	return description;
}

- (NSArray *)update:(NSString *)status
{
	return [self update:status withLocation:nil];
}

- (NSArray *)update:(NSString *)status
	   withLocation:(NSString *)location
{
	NSString *aLocation = @"";
	if (location && ![location isEqualToString:@""])
	{
		aLocation = [NSString stringWithFormat:@"&location=%@",
					 [HTTP urlEncode:location
						   unescaped:nil
							 escaped:@"&+"]];
	}
	NSArray *res = [self postWithAuth:@"update.json"
							   header:[[[NSMutableDictionary alloc] initWithObjectsAndKeys:
										@"application/x-www-form-urlencoded",
										@"Content-Type",
										nil]
									   autorelease]
								 body:[NSString stringWithFormat:
									   @"method=presence.send&message=%@%@",
									   [HTTP urlEncode:status
											 unescaped:nil
											   escaped:@"&+"],
									   aLocation]];
	NSError *error = (NSError *)[NSNull null];
	NSURLResponse *response = (NSURLResponse *)[NSNull null];
	id result = [NSNull null];
	if ([res count] > ResultError)
	{
		error = [res objectAtIndex:ResultError];
	}
	if ([res count] > ResultResponse)
	{
		response = [res objectAtIndex:ResultResponse];
	}
	if ([res count] > ResultData)
	{
		NSData *data = [res objectAtIndex:ResultData];
		if (![data isKindOfClass:[NSNull class]])
		{
			result = [[NSString alloc] initWithData:data
										   encoding:NSUTF8StringEncoding];
		}
	}
	if ([error class] == [NSNull class])
	{
		@try
		{
			result = [result JSONValue];
		}
		@catch (NSException *exception)
		{
			NSData *data = [res objectAtIndex:ResultData];
			result = [[NSString alloc] initWithData:data
										   encoding:NSUTF8StringEncoding];
			NSLog(@"[%@ update] %@: %@", [self className], [exception name], [exception reason]);
		}
	}
	NSString *description = [self determineError:(NSError *)error
										response:(NSURLResponse *)response
										  result:(id)result];
	if (!description)
	{
		description = (NSString *)[NSNull null];
	}
	return [NSArray arrayWithObjects:
			error,
			response,
			result,
			description,
			nil];
}
@end
